@@ -3,6 +3,11 @@
 
  */
 
+#include <openssl/aes.h>
+#include <openssl/evp.h>
+#include <openssl/sha.h>
+
+#include "utils.h"
 #include "Constants.h"
 #include "DatabaseManager.h"
 #include "eWallet.h"
@@ -13,8 +18,7 @@
 #include <QFile>
 #include <QDesktopServices>
 #include <QDebug>
-#include <openssl/evp.h>
-#include <openssl/sha.h>
+
 
 
 #define nBASE64
@@ -84,8 +88,19 @@ int Target::getFieldByName (QString name) {
 	return -1;
 }
 
+QString Target::getName () {
+	return name;
+//	QString tmp = "<qt>%1<br>%2 %3</qt>";
+//	tmp = tmp.arg(this->name).arg(this->creationdate).arg(this->modificationdate);
+////	//return this->type;
+//	return tmp;
+}
+
 QString Target::getSummary () {
+//	QString tmp;
+//	tmp << this->type << " " << this->creationdate << " " << this->modificationdate;
 	return this->type;
+//	return tmp;
 }
 QImage * Target::getPictureSmall () {
 	////qDebug () << "getPictureSmall " << this->id << " " << pictureSmall;
@@ -156,11 +171,24 @@ DatabaseManager::~DatabaseManager()
 
 	//qDebug () << "CLOSE DB " << connectionName;
 	if (db.isOpen()) {
-		//qDebug () << "CLOSing DB";
+		qDebug () << "CLOSing DB " << connectionName;
 		db.close();
 	}
+	opendb = false;
 }
 
+bool DatabaseManager::close()
+{
+	if (db.isOpen()) {
+		qDebug () << "CLOSing DB " << connectionName;
+		db.close();
+	}
+
+	opendb = false;
+	return true;
+}
+
+
 bool DatabaseManager::isInit()
 {
 	//qDebug () << "is init?";
@@ -193,7 +221,7 @@ bool DatabaseManager::initDB()
 bool DatabaseManager::migrateVersion (int fromV, int toV, const QString & pas) {
 	bool ret = true;
 
-	//qDebug () << "MIGRATING DB VERSION FROM " << fromV << " TO " << toV;
+	qDebug () << "MIGRATING DB VERSION FROM " << fromV << " TO " << toV;
 
 	if (fromV == 0 && toV == 1) {
 		if (backupDB ("BACK")) {
@@ -218,6 +246,26 @@ bool DatabaseManager::migrateVersion (int fromV, int toV, const QString & pas) {
 			ret = false;
 	}
 
+	if (fromV == 2 && toV == 3) {
+		if (backupDB ("BACK")) {
+			QSqlQuery query(db);
+			qDebug () << "query 0 " << ret;
+			ret &= query.exec("alter table targets ADD COLUMN creationdate varchar(24)");
+			qDebug () << "query 1 " << ret << " " << query.lastError();
+			ret &= query.exec("alter table targets ADD COLUMN modificationdate varchar(24)");
+			qDebug () << "query 2 " << ret << " " << query.lastError();
+
+			ret &= query.exec("update targets set creationdate = CURRENT_TIMESTAMP, modificationdate = CURRENT_TIMESTAMP");
+			qDebug () << "query 3 " << ret << " " << query.lastError();
+
+			if (ret && !addVersion(3)) {
+				ret = false;
+			}
+		}
+		else
+			ret = false;
+	}
+
 	//qDebug () << "MIGRATED DB VERSION FROM " << fromV << " TO " << toV << " WITH RESULT " << ret;
 
 	return ret;
@@ -229,17 +277,20 @@ bool DatabaseManager::deleteDB(const QString & prefix)
 	db.close();
 
 	QString path(QDir::home().path());
-	path.append(QDir::separator()).append(prefix).append("qtwallet.db.sqlite");
+	//path.append(QDir::separator()).append(FOLDER);
+	path.append(QDir::separator()).append(QDir::separator()).append(prefix).append("qtwallet.db.sqlite");
 	path = QDir::toNativeSeparators(path);
 
 	return QFile::remove(path);
 }
 bool DatabaseManager::backupDB(const QString & prefix) {
 	QString path(QDir::home().path());
+	//path.append(QDir::separator()).append(FOLDER);
 	path.append(QDir::separator()).append("qtwallet.db.sqlite");
 	path = QDir::toNativeSeparators(path);
 
 	QString pathBack(QDir::home().path());
+	//pathBack.append(QDir::separator()).append(FOLDER);
 	pathBack.append(QDir::separator()).append(prefix).append("qtwallet.db.sqlite");
 	pathBack = QDir::toNativeSeparators(pathBack);
 
@@ -254,16 +305,18 @@ bool DatabaseManager::backupDB(const QString & prefix) {
 		dup = file.copy(pathBack);
 	}
 
-	//qDebug () << "duplicate " << prefix << " DB " << dup;
+	qDebug () << "duplicate " << prefix << " DB " << dup;
 
 	return dup;
 }
 bool DatabaseManager::restoreDB(const QString & prefix) {
 	QString path(QDir::home().path());
+	//path.append(QDir::separator()).append(FOLDER);
 	path.append(QDir::separator()).append("qtwallet.db.sqlite");
 	path = QDir::toNativeSeparators(path);
 
 	QString pathBack(QDir::home().path());
+	//pathBack.append(QDir::separator()).append(FOLDER);
 	pathBack.append(QDir::separator()).append(prefix).append("qtwallet.db.sqlite");
 	pathBack = QDir::toNativeSeparators(pathBack);
 
@@ -340,6 +393,7 @@ bool DatabaseManager::_openDB(const QString & name, const QString & cName) {
 		db = QSqlDatabase::addDatabase("QSQLITE");
 	// http://doc.trolltech.com/4.5/sql-driver.html#qsqlite
 	QString path(QDir::home().path());
+	//path.append(QDir::separator()).append(FOLDER);
 	path.append(QDir::separator()).append(name);
 	path = QDir::toNativeSeparators(path);
 
@@ -393,6 +447,8 @@ int DatabaseManager::getVersion() {
 	return -1;
 }
 bool DatabaseManager::addVersion(int v) {
+	qDebug () << "add new version " << v;
+
 	QSqlQuery query (db);
 	bool ret = query.prepare("insert into softwareVersion values (:ver)");
 	if (ret) {
@@ -456,7 +512,8 @@ QString DatabaseManager::changePassword (const QString & newPassword) {
 				if (ret) {
 					if (oldDB->migrationFrom(this)) {
 						//qDebug () << "succesfull migrated ";
-						db.close ();
+						close ();
+						oldDB->close();
 
 						ret = oldDB->restoreDB ("NEW");
 						if (ret) {
@@ -520,9 +577,14 @@ bool DatabaseManager::openDB(const QString & pas) {
 		bool res = false;
 		// Check that tables exists
 		if (db.tables().count() == 3) {
+#ifdef Q_WS_MAEMO_5
+			show_banner(NULL, tr("Migrating database from version %1, please wait").arg(0));
+#endif
 			res = migrateVersion (0, 1, pas);
-			res = createTableVersion ();
-			addVersion (1);
+			if (res)
+				res = createTableVersion ();
+			if (res)
+				res = addVersion (1);
 			currentVersion = 1;
 		}
 
@@ -533,8 +595,30 @@ bool DatabaseManager::openDB(const QString & pas) {
 			currentVersion = SOFT_VERSION;
 		}
 		else {
-			res = migrateVersion (getVersion (), SOFT_VERSION, pas);
 			currentVersion = getVersion ();
+			if (currentVersion < SOFT_VERSION) {
+				do {
+#ifdef Q_WS_MAEMO_5
+			show_banner(NULL, tr("Migrating database from version %1, please wait").arg(currentVersion));
+#endif
+					res = migrateVersion (currentVersion, currentVersion + 1, pas);
+					if (res)
+						currentVersion =  getVersion ();
+				}
+				while (res && currentVersion < SOFT_VERSION);
+			}
+			else
+				res = true;
+
+
+			if (currentVersion != SOFT_VERSION || !res) {
+				// TODO show an error!!!
+#ifdef Q_WS_MAEMO_5
+		show_information_note (NULL, tr("Migration not possible, stop at version %1").arg(currentVersion));
+#endif
+				qDebug () << "MIGRATION NOT POSSIBLE " << currentVersion << " to " << SOFT_VERSION;
+				return false;
+			}
 
 			if (hasSecretPhrase()) {
 				initdb = checkPassword(pas, currentVersion);
@@ -571,6 +655,10 @@ bool DatabaseManager::checkPassword (QString _a, int version)
 	/*unsigned char key16[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
 	unsigned char key24[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23};
 	unsigned char key32[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31};*/
+
+//	int AES_set_encrypt_key(const unsigned char *userKey, const int bits,
+//		AES_KEY *key);
+
 	AES_set_encrypt_key(key, keyLen, &aeskeyE);
 	AES_set_decrypt_key(key, keyLen, &aeskeyD);
 	
@@ -677,7 +765,10 @@ bool DatabaseManager::createTargetTable()
 		                 "name varchar(30), "
 		                 "picture BLOB, "
 		                 "pictureBack BLOB, "
-		                 "type varchar(5), groupName varchar(100))");
+		                 "type varchar(5), "
+		                 "groupName varchar(100), "
+		                 "creationdate varchar(24), "
+		                 "modificationdate varchar(24))");
 
 	}
 	return ret;
@@ -825,7 +916,7 @@ bool DatabaseManager::insertTarget(Target* target)
 		// http://www.sqlite.org/autoinc.html
 		// NULL = is the keyword for the autoincrement to generate next value
 		QSqlQuery query (db);
-		ret = query.prepare("INSERT INTO targets (name, type, picture, pictureBack, groupName) VALUES (:name, :type, :picture, :pictureBack, :groupName)");
+		ret = query.prepare("INSERT INTO targets (name, type, picture, pictureBack, groupName, creationdate, modificationdate) VALUES (:name, :type, :picture, :pictureBack, :groupName, CURRENT_TIMESTAMP, CURRENT_TIMESTAMP)");
 		QByteArray t1;
 
 		//qDebug() << "step 1" ;
@@ -887,7 +978,7 @@ bool DatabaseManager::copyTarget(Target* target)
 	bool ret = false;
 	if (db.isOpen()) {
 		QSqlQuery query (db);
-		ret = query.prepare("INSERT INTO targets (id, name, type, picture, pictureBack, groupName) VALUES (:id, :name, :type, :picture, :pictureBack, :groupName)");
+		ret = query.prepare("INSERT INTO targets (id, name, type, picture, pictureBack, groupName, creationdate, modificationdate) VALUES (:id, :name, :type, :picture, :pictureBack, :groupName, :creationdate, :modificationdate)");
 		QByteArray t1;
 
 		if (ret) {
@@ -896,6 +987,8 @@ bool DatabaseManager::copyTarget(Target* target)
 			query.bindValue(":name", t1);
 			query.bindValue(":type", target->type);
 			query.bindValue(":groupName", target->groupName);
+			query.bindValue(":creationdate", target->creationdate);
+			query.bindValue(":modificationdate", target->modificationdate);
 					//qDebug() << "step 4" ;
 
 			QByteArray im1;
@@ -938,11 +1031,11 @@ QList<Target*> DatabaseManager::getTargets(const QString * filter)
 	QSqlQuery query (db);
 	bool ret;
 	if (filter != NULL) {// && filter.length() > 0) {
-		ret = query.prepare("select id, name, type, groupName from targets where coalesce(groupName,'') = :groupName");
+		ret = query.prepare("select id, name, type, groupName, creationdate, modificationdate from targets where coalesce(groupName,'') = :groupName");
 		query.bindValue(":groupName", *filter);
 	}
 	else {
-		ret = query.prepare("select id, name, type, groupName from targets");
+		ret = query.prepare("select id, name, type, groupName, creationdate, modificationdate from targets");
 	}
 
 	ret = query.exec ();
@@ -956,7 +1049,11 @@ QList<Target*> DatabaseManager::getTargets(const QString * filter)
 		item->pictureSmall = NULL;
 		item->picture = NULL;
 		item->pictureBack = NULL;
+		item->creationdate = query.value(4).toString();
+		item->modificationdate = query.value(5).toString();
 		result.append(item);
+
+		 QApplication::instance()->processEvents();
 	}
 
 	return result;
@@ -1047,7 +1144,7 @@ bool DatabaseManager::updateTarget(Target* target)
 	bool ret = false;
 	if (db.isOpen()) {
 		QSqlQuery query (db);
-		ret = query.prepare("UPDATE targets set name=:name, type=:type, groupName=:groupName where id=:code");
+		ret = query.prepare("UPDATE targets set name=:name, type=:type, groupName=:groupName, modificationdate=CURRENT_TIMESTAMP where id=:code");
 		QByteArray t1;
 		if (ret) {
 			encrypt(target->name, t1);
@@ -1069,7 +1166,7 @@ bool DatabaseManager::updateTargetPicture(Target* target)
 	bool ret = false;
 	if (db.isOpen()) {
 		QSqlQuery query (db);
-		ret = query.prepare("UPDATE targets set picture=:picture where id=:code");
+		ret = query.prepare("UPDATE targets set picture=:picture, modificationdate=CURRENT_TIMESTAMP where id=:code");
 		if (ret) {
 			//qDebug () << "updating image " << target->id << " " << target->picture;
 
@@ -1105,7 +1202,7 @@ bool DatabaseManager::updateTargetPictureBack(Target* target)
 	bool ret = false;
 	if (db.isOpen()) {
 		QSqlQuery query (db);
-		ret = query.prepare("UPDATE targets set pictureBack=:pictureBack where id=:code");
+		ret = query.prepare("UPDATE targets set pictureBack=:pictureBack, modificationdate=CURRENT_TIMESTAMP where id=:code");
 		if (ret) {
 			//qDebug () << "UPDATE " << target->id << " " << target->name << " " << target->type;
 			QByteArray im2;
@@ -1328,7 +1425,7 @@ bool DatabaseManager::renameGroup (const QString & from, const QString & to) {
 	bool ret = false;
 	if (db.isOpen()) {
 		QSqlQuery query (db);
-		ret = query.prepare("UPDATE targets set groupName = :newGroupName WHERE coalesce(groupName,'') = :oldGroupName");
+		ret = query.prepare("UPDATE targets set groupName = :newGroupName, modificationdate=CURRENT_TIMESTAMP WHERE coalesce(groupName,'') = :oldGroupName");
 		if (ret) {
 			query.bindValue(":newGroupName", to);
 			query.bindValue(":oldGroupName", from);
@@ -1342,7 +1439,7 @@ bool DatabaseManager::deleteGroup (const QString & from) {
 	bool ret = false;
 	if (db.isOpen()) {
 		QSqlQuery query (db);
-		ret = query.prepare("UPDATE targets set groupName = '' WHERE coalesce(groupName,'') = :oldGroupName");
+		ret = query.prepare("UPDATE targets set groupName = '', modificationdate=CURRENT_TIMESTAMP WHERE coalesce(groupName,'') = :oldGroupName");
 		if (ret) {
 			query.bindValue(":oldGroupName", from);
 			ret = query.exec ();
