@@ -81,6 +81,9 @@ public:
 	QString groupName;
 	QString type;
 
+	QString creationdate;
+	QString modificationdate;
+
 	QImage * getPictureSmall ();
 	QImage * getPicture ();
 	QImage * getPictureBack ();
@@ -107,6 +110,8 @@ public:
 	QList<TargetField*> fields; // ?? * or not
 
 	bool listLoaded;
+
+	QString getName ();
 };
 
 
@@ -124,6 +129,7 @@ public:
 	//bool sortTargets (const Target * f, const Target * g);
 
 	bool openDB(const QString & pass);
+	bool close ();
 	bool hasPassword ();
 	bool isReady ();
 	void lock ();
