@@ -4,7 +4,7 @@
  *
  */
 
-#include <stdlib.h>
+//#include <stdlib.h>
 
 #include "utils.h"
 
@@ -23,41 +23,57 @@
 #include "camera.h"
 #endif
 
-class QLabelW: public QLabel {
+QLabelW::QLabelW(const QString &text, bool isLabel, QWidget *parent, Qt::WindowFlags f) : QLabel (text, parent, f) {
+	this->isLabel = isLabel;
+	setContextMenuPolicy(Qt::CustomContextMenu);
+}
 
-public:
-	QLabelW(const QString &text, bool isLabel, QWidget *parent=0, Qt::WindowFlags f=0) : QLabel (text, parent, f) {
-		this->isLabel = isLabel;
+QLabelW::QLabelW(QWidget *parent, Qt::WindowFlags f): QLabel (parent, f) {
+}
+
+QLabelW::~QLabelW() {
+}
+
+void QLabelW::m_copy () {
+   	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+
+#ifdef Q_WS_MAEMO_5
+	show_banner(this, tr("%1 has been saved to clipboard").arg(f->name));
+#endif
+	QClipboard *cb = QApplication::clipboard();
+	cb->setText(f->value);
+}
+
+void QLabelW::m_goto () {
+	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+	if (f->type == TYPE_URL && !isLabel) {
+		QDesktopServices::openUrl(QUrl::fromUserInput(f->value));
 	}
-	QLabelW(QWidget *parent=0, Qt::WindowFlags f=0): QLabel (parent, f) {
-	}
-	~QLabelW() {
-	}
-protected:
-	bool isLabel;
-    void mousePressEvent(QMouseEvent *) {
-    	////qDebug () << "click on field " << this->userData(0);
-    	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
-    	if (f->type == TYPE_URL && !isLabel) {
-    		QDesktopServices::openUrl(QUrl::fromUserInput(f->value));
-    	}
-    	else {
-    		if (!isLabel) {
-    			if (f->type == TYPE_PHONE_NUMBER) {
+}
+
+void QLabelW::m_sms () {
+
+}
+
+void QLabelW::m_call () {
+	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+	if (!isLabel) {
+		if (f->type == TYPE_PHONE_NUMBER) {
 //    				QSettings settings;
 
-    				QString telf = f->value;
-    				telf = telf.simplified();
-    				telf = telf.replace(" ", "");
-
-    				if (telf.length() > 0) {
-    					qDebug () << "calling to " << telf;
-						QDBusConnection conn = QDBusConnection::systemBus(); //QString(MCE_SERVICE), QString(MCE_SIGNAL_PATH)
-						QDBusInterface dbusPhone( QString ("com.nokia.csd.Call"), QString ("/com/nokia/csd/call"), QString ("com.nokia.csd.Call"), conn );
-						QDBusMessage msg = dbusPhone.call( QString ("CreateWith"), telf, (uint)0 );
-						qDebug() << msg;
-    				}
+			QString telf = f->value;
+			telf = telf.simplified();
+			telf = telf.replace(" ", "");
 
+			if (telf.length() > 0) {
+				qDebug () << "calling to " << telf;
+				QDBusConnection conn = QDBusConnection::systemBus(); //QString(MCE_SERVICE), QString(MCE_SIGNAL_PATH)
+				QDBusInterface dbusPhone( QString ("com.nokia.csd.Call"), QString ("/com/nokia/csd/call"), QString ("com.nokia.csd.Call"), conn );
+				QDBusMessage msg = dbusPhone.call( QString ("CreateWith"), telf, (uint)0 );
+				qDebug() << msg;
+			}
+		}
+	}
 //    				if (settings.value("TT/method1").toBool()) {
 //    					qDebug () << "method1";
 //
@@ -93,25 +109,28 @@ protected:
 //
 //    					::system ((const char *)(QString ("dbus-send --system --type=method_call --dest=com.nokia.csd.Call /com/nokia/csd/call com.nokia.csd.Call.CreateWith string:\"%1\" uint32:0").arg(f->value).constData()));
 //    				}
-    			}
-    			else {
-#ifdef Q_WS_MAEMO_5
-    				show_banner(this, tr("%1 has been saved to clipboard").arg(f->name));
-#endif
-					QClipboard *cb = QApplication::clipboard();
-					cb->setText(f->value);
-    			}
-    		}
-    		else {
-#ifdef Q_WS_MAEMO_5
-    			show_banner(this, tr("%1 has been saved to clipboard").arg(f->name));
-#endif
- 				QClipboard *cb = QApplication::clipboard();
-				cb->setText(f->value);
-    		}
-    	}
-    }
-};
+}
+
+void QLabelW::mousePressEvent(QMouseEvent *) {
+	////qDebug () << "click on field " << this->userData(0);
+	TargetField * f = ((ButtonInfo *)this->userData(0))->field;
+
+	if (isLabel) {
+		m_copy ();
+	}
+//	else
+//	if (f->type == TYPE_URL) {
+//		m_goto ();
+//	}
+//	else
+//	if (f->type == TYPE_PHONE_NUMBER) {
+//		m_call ();
+//	}
+//	else {
+//		m_copy ();
+//   	}
+ }
+
 
 DetailWindow::DetailWindow(Target * _target, const QString & groupName, int istemplate, bool edit, eWallet *paTarget, QWidget *parent) :
 		QMainWindow(parent)
@@ -200,8 +219,10 @@ DetailWindow::DetailWindow(Target * _target, const QString & groupName, int iste
 			menuBar()->addAction(tr("Delete"), this, SLOT(_deleteTarget()));
 		menuBar()->addAction(tr("Change Face Pict."), this, SLOT(cardFace()));
 		menuBar()->addAction(tr("Mirror Face Pict."), this, SLOT(mirrorFace()));
+		menuBar()->addAction(tr("Rotate Face Pict."), this, SLOT(rotateFace()));
 		menuBar()->addAction(tr("Change Back Pict."), this, SLOT(cardBack()));
 		menuBar()->addAction(tr("Mirror Back Pict."), this, SLOT(mirrorBack()));
+		menuBar()->addAction(tr("Rotate Back Pict."), this, SLOT(rotateBack()));
 		menuBar()->addAction(tr("Delete Face Pict."), this, SLOT(deleteCardFace()));
 		menuBar()->addAction(tr("Delete Back Pict."), this, SLOT(deleteCardBack()));
 		menuBar()->addAction(tr("Modify fields"), this, SLOT(modifyFields()));
@@ -213,10 +234,12 @@ DetailWindow::DetailWindow(Target * _target, const QString & groupName, int iste
 		m = menuBar()->addMenu(tr("Pictures"));
 		m->addAction(tr("Change Face Pict."), this, SLOT(cardFace()));
 		m->addAction(tr("Mirror Face Pict."), this, SLOT(mirrorFace()));
+		m->addAction(tr("Rotate Face Pict."), this, SLOT(rotateFace()));
 		m->addAction(tr("Delete Face Pict."), this, SLOT(deleteCardFace()));
 		m->addSeparator ();
 		m->addAction(tr("Change Back Pict."), this, SLOT(cardBack()));
 		m->addAction(tr("Mirror Back Pict."), this, SLOT(mirrorBack()));
+		m->addAction(tr("Rotate Back Pict."), this, SLOT(rotateBack()));
 		m->addAction(tr("Delete Back Pict."), this, SLOT(deleteCardBack()));
 		m = menuBar()->addMenu(tr("Others"));
 		m->addAction(tr("Modify fields"), this, SLOT(modifyFields()));
@@ -704,12 +727,17 @@ void DetailWindow::showFields () {
 							nameLineEdit->setWordWrap(true);
 							componentLayout->addWidget(nameLineEdit, pos, 1, 1, 1);
 							//componentLayout->setRowMinimumHeight(pos, 300);
+							//QObject::connect(nameLineEdit, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(labelMenu(const QPoint&)));
 						}
 						else {
 							QLabel * nameLineEdit = new QLabelW(field->value, false, this);
 							nameLineEdit->setUserData(0, new ButtonInfo (field));
 							nameLineEdit->setWordWrap(true);
 							componentLayout->addWidget(nameLineEdit, pos, 1, 1, 1);
+
+
+							//QObject::connect(nameLineEdit, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(labelMenu(const QPoint&)));
+
 						}
 					}
 				}
@@ -964,6 +992,12 @@ void DetailWindow::loadImage (QString q, QImage& res) {
 	////qDebug () << "read " << q << " as (" << imageSize << ") with result " << reader.errorString ();
 }
 
+void DetailWindow::labelMenu(const QPoint & _p) {
+	QMenu menu(this);
+	menu.addAction(tr("View"), this, SLOT(cardFace()));
+	menu.exec(_p);
+}
+
 void DetailWindow::cardMenu(const QPoint & _p) {
 	QPoint p = _p;//this->mapFromGlobal(_p);
 
@@ -972,6 +1006,9 @@ void DetailWindow::cardMenu(const QPoint & _p) {
 	if (bar)
 		p.setY(p.y() + bar->value());
 
+	QWidget * tmp = centralWidget->childAt(p);
+	//qDebug () << tmp;
+
 	entryScrollArea->ensureVisible(p.x(), p.y());
 
 	if (this->imageLabel != NULL && this->imageLabel->geometry().contains(p)) {
@@ -979,6 +1016,7 @@ void DetailWindow::cardMenu(const QPoint & _p) {
 		if (edit) {
 			menu.addAction(tr("Edit"), this, SLOT(cardFace()));
 			menu.addAction(tr("Mirror"), this, SLOT(mirrorFace()));
+			menu.addAction(tr("Rotate"), this, SLOT(rotateFace()));
 			menu.addAction(tr("Delete"), this, SLOT(deleteCardFace()));
 		}
 		else {
@@ -992,6 +1030,7 @@ void DetailWindow::cardMenu(const QPoint & _p) {
 		if (edit) {
 			menu.addAction(tr("Edit"), this, SLOT(cardBack()));
 			menu.addAction(tr("Mirror"), this, SLOT(mirrorBack()));
+			menu.addAction(tr("Rotate"), this, SLOT(rotateBack()));
 			menu.addAction(tr("Delete"), this, SLOT(deleteCardBack()));
 		}
 		else {
@@ -999,6 +1038,31 @@ void DetailWindow::cardMenu(const QPoint & _p) {
 		}
 		menu.exec(_p);
 	}
+	else {
+		if (tmp != NULL) {
+			if (tmp->userData(0) != NULL) {
+				TargetField * f = ((ButtonInfo *)tmp->userData(0))->field;
+
+				QMenu menu(this);
+		    	if (f->type == TYPE_URL) {
+					menu.addAction(tr("Go to"), tmp, SLOT(m_goto()));
+					menu.addAction(tr("Copy"), tmp, SLOT(m_copy()));
+		    		//QDesktopServices::openUrl(QUrl::fromUserInput(f->value));
+		    	}
+		    	else {
+	    			if (f->type == TYPE_PHONE_NUMBER) {
+	    				menu.addAction(tr("Call to"), tmp, SLOT(m_call()));
+	    				//menu.addAction(tr("SMS to"), tmp, SLOT(m_sms()));
+	    				menu.addAction(tr("Copy"), tmp, SLOT(m_copy()));
+	    			}
+	    			else {
+	    				menu.addAction(tr("Copy"), tmp, SLOT(m_copy()));
+	    			}
+		    	}
+				menu.exec(_p);
+			}
+		}
+	}
 }
 void DetailWindow::deleteCardFace()
 {
@@ -1058,6 +1122,29 @@ void DetailWindow::cardFace()
 	}
 }
 
+void DetailWindow::rotateFace () {
+	if (currentImageN) {
+		QImage * tmp = currentImageN;
+
+		QMatrix m;
+		m.rotate(90);
+		currentImageN = new QImage (currentImageN->transformed(m));
+
+		imageLabel->setPixmap(QPixmap::fromImage (*currentImageN));
+		delete tmp;
+		deleteFace = false;
+	}
+	else
+	if (currentImage) {
+		QMatrix m;
+		m.rotate(90);
+		currentImageN = new QImage (currentImage->transformed(m));
+
+		imageLabel->setPixmap(QPixmap::fromImage (*currentImageN));
+		deleteFace = false;
+	}
+}
+
 void DetailWindow::mirrorFace () {
 	if (currentImageN) {
 		QImage * tmp = currentImageN;
@@ -1089,6 +1176,26 @@ void DetailWindow::mirrorBack () {
 		deleteBack = false;
 	}
 }
+
+void DetailWindow::rotateBack () {
+	if (currentImageBackN) {
+		QImage * tmp = currentImageBackN;
+		QMatrix m;
+		m.rotate(90);
+		currentImageBackN = new QImage (tmp->transformed (m));
+		imageBackLabel->setPixmap(QPixmap::fromImage (*currentImageBackN));
+		delete tmp;
+		deleteBack = false;
+	}
+	else
+	if (currentImageBack) {
+		QMatrix m;
+		m.rotate(90);
+		currentImageBackN = new QImage (currentImageBack->transformed (m));
+		imageBackLabel->setPixmap(QPixmap::fromImage (*currentImageBackN));
+		deleteBack = false;
+	}
+}
 //void DetailWindow::contextMenuEvent (QContextMenuEvent * event) {
 //	//qDebug () << "NOW";
 //}
