@@ -114,6 +114,7 @@ public slots:
 
 	void cardFace();
 	void mirrorFace();
+	void rotateFace();
 
 	void orientationChanged (int);
 
@@ -121,10 +122,12 @@ public slots:
 	void exportCardFace();
 	void cardBack();	
 	void mirrorBack();
+	void rotateBack();
 	void deleteCardBack();
 	void exportCardBack();
 
 	void cardMenu(const QPoint & pos);
+	void labelMenu(const QPoint & pos);
 //	void cardFaceMenu(const QPoint & pos);
 //	void cardBackMenu(const QPoint & pos);
 //	void contextMenuEvent (QContextMenuEvent * event);
@@ -155,4 +158,20 @@ public:
 	}
 };
 
+class QLabelW: public QLabel {
+	Q_OBJECT
+public:
+	QLabelW(const QString &text, bool isLabel, QWidget *parent=0, Qt::WindowFlags f=0);
+	QLabelW(QWidget *parent=0, Qt::WindowFlags f=0);
+	~QLabelW();
+public slots:
+	void m_copy ();
+	void m_goto ();
+	void m_sms ();
+	void m_call ();
+protected:
+	bool isLabel;
+    void mousePressEvent(QMouseEvent *);
+};
+
 #endif // DETAIL_H
