@@ -6,15 +6,17 @@
 // TODO modify picture (zoom, ...)
  * TODO after change type it is not refreshed on list
  * TODO implements copy of information field (and images?)
- * TODO use hilton notifications and banner???
  * TODO check types on import
  * TODO sort after insert new card
  * TODO pc version?
  * TODO fix problems with key
  * TODO after import groups are not refresh
- * TODO move language files outside resources
- * TODO export image depends on extension selected
- */
+ * TODO get language from system not from regional settings
+ * TODO export to xml
+ * TODO import from xml
+ * TODO move fields value into a big field
+ * TODO what happens with small pictures
+  */
 
 #include "utils.h"
 #include <QDBusInterface>
@@ -35,9 +37,6 @@
 //#include <hildonmm.h>
 //#include <gtk.h>//hildon-notify/hildon/hildon-notification.h>
 
-
-
-
 //#ifdef Q_WS_MAEMO_5
 //#include <mce/mode-names.h>
 //#include <mce/dbus-names.h>
@@ -135,7 +134,7 @@ eWallet::eWallet(QString * group, eWallet * _parent, QWidget *parent) : QMainWin
 
 	isShowGroups = 2;
 
-	showList ();
+	showList (true);
 	showTargets(true, currentGroup);
 }
 
@@ -184,7 +183,7 @@ void eWallet::orientationChanged(int o) {
 	}
 }
 
-void eWallet::showList () {
+void eWallet::showList (bool interactive) {
 	//qDebug () << "going to showList";
 
 	isAskPassword = false;
@@ -201,7 +200,12 @@ void eWallet::showList () {
 
 	//backButton = new QPushButton("<< Back", this);
 
-	setContextMenuPolicy(Qt::DefaultContextMenu);
+	if (interactive) {
+		setContextMenuPolicy(Qt::DefaultContextMenu);
+	}
+	else {
+		setContextMenuPolicy(Qt::NoContextMenu);
+	}
 //	centralWidget->setContextMenuPolicy(Qt::DefaultContextMenu);
 
 	vboxlayout = new QVBoxLayout;
@@ -217,33 +221,42 @@ void eWallet::showList () {
 
 	//fnListWidget->grabGesture(Qt::SwipeGesture);
 
-	QObject::connect(fnListWidget, SIGNAL(itemClicked(QListWidgetItem *)), this, SLOT(selectItem(QListWidgetItem *)));
-	QObject::connect(fnListWidget, SIGNAL(itemDoubleClicked(QListWidgetItem *)), this, SLOT(selectDItem(QListWidgetItem *)));
+	if (interactive) {
+		QObject::connect(fnListWidget, SIGNAL(itemClicked(QListWidgetItem *)), this, SLOT(selectItem(QListWidgetItem *)));
+		QObject::connect(fnListWidget, SIGNAL(itemDoubleClicked(QListWidgetItem *)), this, SLOT(selectDItem(QListWidgetItem *)));
 
-	inputWidget = new QHBoxLayout;
-	inputWidget->setContentsMargins(2, 2, 2, 0);
-	vboxlayout->addLayout(inputWidget);
+		inputWidget = new QHBoxLayout;
+		inputWidget->setContentsMargins(2, 2, 2, 0);
+		vboxlayout->addLayout(inputWidget);
 
-	cancelSearch = new QPushButton(QIcon(":/resources/icon_clear.png"), "", this);
-	searchField = new QLineEdit ();
-	hideSearch = new QPushButton(QIcon(":/resources/icon_hide.png"), "", this);
+		cancelSearch = new QPushButton(QIcon(":/resources/icon_clear.png"), "", this);
+		searchField = new QLineEdit ();
+		hideSearch = new QPushButton(QIcon(":/resources/icon_hide.png"), "", this);
 
-	inputWidget->addWidget(cancelSearch);
-	searchField->setText(currentFilter);
-	searchField->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
-	inputWidget->addWidget(searchField);
-	inputWidget->addWidget(hideSearch);
+		inputWidget->addWidget(cancelSearch);
+		searchField->setText(currentFilter);
+		searchField->setInputMethodHints(Qt::ImhNoAutoUppercase | Qt::ImhNoPredictiveText);
+		inputWidget->addWidget(searchField);
+		inputWidget->addWidget(hideSearch);
 
-	//QObject::connect(backButton, SIGNAL(clicked()), this, SLOT(_clickBack()));
-	//backButton->hide ();
+		//QObject::connect(backButton, SIGNAL(clicked()), this, SLOT(_clickBack()));
+		//backButton->hide ();
 
-	QObject::connect(cancelSearch, SIGNAL(clicked()), this, SLOT(_cancelSearch()));
-	QObject::connect(hideSearch, SIGNAL(clicked()), this, SLOT(_hideSearch()));
-	QObject::connect(searchField, SIGNAL(textChanged(const QString &)), this, SLOT(textChanged(const QString &)));
+		QObject::connect(cancelSearch, SIGNAL(clicked()), this, SLOT(_cancelSearch()));
+		QObject::connect(hideSearch, SIGNAL(clicked()), this, SLOT(_hideSearch()));
+		QObject::connect(searchField, SIGNAL(textChanged(const QString &)), this, SLOT(textChanged(const QString &)));
+	}
+	else {
+		searchField = NULL;
+		cancelSearch = NULL;
+		hideSearch = NULL;
+	}
 
 	centralWidget->setLayout(vboxlayout);
 
-	createMenu ();
+	if (interactive) {
+		createMenu ();
+	}
 
 	////qDebug () << "setCentralWidget(centralWidget);";
 	setCentralWidget(centralWidget);
@@ -252,18 +265,20 @@ void eWallet::showList () {
 	centralWidget->setFocus ();
 	//fnListWidget->setFocus ();
 
-	if (currentFilter.length () > 0) {
-		cancelSearch->show ();
-		searchField->show ();
-		//setFocus ();
-		//focusNextChild();
-		searchField->setFocus();
-		hideSearch->show ();
-	}
-	else {
-		cancelSearch->hide ();
-		searchField->hide ();
-		hideSearch->hide ();
+	if (interactive) {
+		if (currentFilter.length () > 0) {
+			cancelSearch->show ();
+			searchField->show ();
+			//setFocus ();
+			//focusNextChild();
+			searchField->setFocus();
+			hideSearch->show ();
+		}
+		else {
+			cancelSearch->hide ();
+			searchField->hide ();
+			hideSearch->hide ();
+		}
 	}
 }
 
@@ -543,7 +558,7 @@ bool eWallet::openDataBase (const QString & pas)
 			}
 		}
 
-		showList ();
+		showList (true);
 		showInfo (true);
 	}
 	return res;
@@ -779,7 +794,7 @@ void eWallet::loadTypesAndFields () {
 void eWallet::keyPressEvent(QKeyEvent* event) {
 	//qDebug()<<"key pressed " << event->key();
 	
-	if (!isAskPassword) {
+	if (!isAskPassword && searchField != NULL) {
 		if (searchField->isHidden()) {
 //			if (event->key () == Qt::Key_Left || event->key () == Qt::Key_Right ) {
 //				changeMode();
@@ -911,6 +926,12 @@ void eWallet::showTargets(bool reset, const QString * filter)
 
 	int current = fnListWidget->currentRow();
 
+//#ifdef Q_WS_MAEMO_5
+//	show_banner (this, tr("Loading data, please wait"));
+//#endif
+
+    //QApplication::instance()->processEvents();
+
 	// Clean previous items from the layout
 	fnListWidget->clear();
 
@@ -930,7 +951,7 @@ void eWallet::showTargets(bool reset, const QString * filter)
 	for (int i = 0; i < list.size (); i++) {
 		Target * target = list.at(i);
 
-		QListWidgetItem* listitem = new QListWidgetItem(getImage(target), target->name, fnListWidget);
+		QListWidgetItem* listitem = new QListWidgetItem(getImage(target), target->getName (), fnListWidget);
 		//listitem->set(target->getSummary());
 		fnListWidget->addItem(listitem);
 		
@@ -969,6 +990,7 @@ void eWallet::createMenu()
 		menuBar()->addAction(tr("Invert Sort"), this, SLOT(sort()));
 		menuBar()->addAction(tr("Import"), this, SLOT(import()));
 		menuBar()->addAction(tr("Export"), this, SLOT(exportData()));
+		//menuBar()->addAction(tr("Export Xml"), this, SLOT(exportXml()));
 		//menuBar()->addAction(tr("Change View"), this, SLOT(changeMode()));
 		menuBar()->addAction(tr("Change Password"), this, SLOT(changePassword()));
 		menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
@@ -994,6 +1016,7 @@ void eWallet::createMenu()
 		m->addSeparator();
 		m->addAction(tr("Import"), this, SLOT(import()));
 		m->addAction(tr("Export"), this, SLOT(exportData()));
+		//m->addAction(tr("Export Xml"), this, SLOT(exportXml()));
 		menuBar()->addAction(tr("Exit"), QApplication::instance(), SLOT(quit()));
 		menuBar()->addAction(tr("About"), this, SLOT(about()));
 		//menuBar()->addAction(tr("Change View"), this, SLOT(changeMode()));
@@ -1595,6 +1618,124 @@ void eWallet::exportData() {
 #endif
 }
 
+QString FIX_NODE (QString a) {
+	// TODO escape characters
+	return a;
+}
+QByteArray FIX_VALUE (QString a) {
+	// TODO escape characters
+	return a.toUtf8();
+}
+
+void eWallet::exportXml() {
+	QString path(QDesktopServices::storageLocation(QDesktopServices::HomeLocation));
+
+	path.append("/wallet_export.xml");
+
+	QString s = QFileDialog::getSaveFileName(
+	                this,
+	                tr("Choose export file"),
+	                path,
+	                tr("Text files (*.xml);;"));
+
+#ifdef Q_WS_MAEMO_5
+	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
+#endif
+	if (!s.isNull () && !s.isEmpty ()) {
+		if (!s.endsWith(".xml"))
+			s.append(".xml");
+
+		  QFile file(s);
+
+		  if ( file.open(QIODevice::Text | QIODevice::WriteOnly) ) {
+		    // file opened successfully
+		    QTextStream t(&file);        // use a text stream
+
+		    QList<Target *> list = ::dbManager->getTargets(NULL);
+
+		    t << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
+			t << "<targets>\n";
+			for (int i = 0; i < list.size (); i++) {
+				Target * target = list[i];
+
+				t << "<target>\n";
+				t << "<name>" << FIX_VALUE(target->name) << "</name>\n";
+				t << "<group>" << FIX_VALUE(target->groupName) << "</group>\n";
+				t << "<type>" << FIX_VALUE(target->type) << "</type>\n";
+
+				QImage * img = target->getPicture();
+
+				if (img != NULL) {
+					QByteArray im1;
+					QBuffer buffer1(&im1);
+					buffer1.open(QIODevice::WriteOnly);
+					img->save(&buffer1, "PNG"); // writes image into ba in PNG format
+
+					t << "<image>" << im1.toBase64() << "</image>\n";
+				}
+
+				img = target->getPictureBack();
+
+				if (img != NULL) {
+					QByteArray im1;
+					QBuffer buffer1(&im1);
+					buffer1.open(QIODevice::WriteOnly);
+					img->save(&buffer1, "PNG"); // writes image into ba in PNG format
+
+					t << "<backimage>" << im1.toBase64() << "</backimage>\n";
+				}
+
+				dbManager->getFields(target);
+
+				TargetField * field;
+				foreach(field, target->fields) {
+	    			  switch (field->type) {
+						  case TYPE_TEXT:
+							t << "<" << FIX_NODE(field->name) << " type=\"T\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+							  break;
+						  case TYPE_DATE:
+								t << "<" << FIX_NODE(field->name) << " type=\"D\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+							  break;
+						  case TYPE_URL:
+								t << "<" << FIX_NODE(field->name) << " type=\"U\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+							  break;
+						  case TYPE_PASSWORD:
+								t << "<" << FIX_NODE(field->name) << " type=\"P\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+							  break;
+						  case TYPE_INTEGER:
+								t << "<" << FIX_NODE(field->name) << " type=\"N\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+							  break;
+						  case TYPE_PHONE_NUMBER:
+								t << "<" << FIX_NODE(field->name) << " type=\"H\">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+							  break;
+						  case 'S':
+						  default:
+								t << "<" << FIX_NODE(field->name) << ">" << FIX_VALUE(field->value) << "</" << FIX_NODE(field->name) << ">\n";
+	    			  }
+				}
+
+				t << "</target>\n";
+
+				target->freeFieldList();
+				target->freeResources();
+
+				QApplication::instance()->processEvents();
+
+				delete target;
+			}
+			t << "</targets>\n";
+
+			list.clear();
+
+		    // Close the file
+		    file.close();
+		  }
+	}
+#ifdef Q_WS_MAEMO_5
+	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
+#endif
+}
+
 void eWallet::editTarget(Target * target) {
 #ifndef Q_WS_MAEMO_5
 	if (!(detailWindow.isNull() && detailWindowEdit.isNull())) {
@@ -1734,7 +1875,7 @@ void eWallet::askPassword () {
 	bool firstTime = !::dbManager->hasPassword ();
 #ifdef Q_WS_MAEMO_5
 	if (firstTime) {
-		show_banner (this, tr("Please, enter a password for your wallet"));
+		show_information_note (this, tr("Please, enter a password for your wallet"));
 	}
 	else {
 		show_banner (this, tr("Please, enter the password to unlock your wallet"));
@@ -1799,7 +1940,7 @@ void eWallet::changePassword () {
 
 
 void eWallet::reallyChangePassword (const QString & newPassword) {
-	showList ();
+	showList (false);
 
 //	QMessageBox * messageBox = new QMessageBox (this);
 //	messageBox->information(this, "Wallet", tr("Changing password, please wait until application quits"), QMessageBox::Ok);
@@ -1846,9 +1987,17 @@ void eWallet::reallyChangePassword (const QString & newPassword) {
 
 	delete messageBox;
 
-	delete ::dbManager;
+	//delete ::dbManager;
+	//exit (3);
+#ifdef Q_WS_MAEMO_5
+	this->setAttribute(Qt::WA_Maemo5ShowProgressIndicator, false);
+#endif
 
-	exit (3);
+	::dbManager->close ();
+	openDataBase (newPassword);
+
+//	::dbManager->lock ();
+//	askPassword ();
 }
 
 
